/*-
 *
 * Copyright (c) 1999 Neil Blakey-Milner
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: lists.c,v 1.6 1999/07/29 17:07:34 nbm Exp $
 *
 */

#include <err.h>
#include <stdlib.h>
#include <string.h>

#include "portconf.h"
#include "lists.h"

struct classList   *
lookup_class(char *name)
{
	int                 q;

	for (q = 0; q < numclasses; q++)
		if (strcasecmp(name, clookup[q]->name) == 0)
			return clookup[q]->cl;

	return NULL;
}

struct optionList  *
lookup_options(char *name)
{
	int                 q;

	for (q = 0; q < numoptions; q++)
		if (strcasecmp(name, olookup[q]->name) == 0)
			return olookup[q]->ol;

	return NULL;
}

void
build_lookup()
{
	struct optionList  *ol;
	struct classList   *cl;
	int                 q;

	q = 0;

	LIST_FOREACH(ol, &ohead, entries) {
		q++;
	}

	numoptions = q;

	if ((olookup = malloc(numoptions * sizeof(struct _olookup *))) == NULL)
		err(1, NULL);

	for (q = 0; q < numoptions; q++)
		if ((olookup[q] = malloc(sizeof(struct _olookup))) == NULL)
			err(1, NULL);

	q = 0;
	LIST_FOREACH(ol, &ohead, entries) {
		olookup[q]->name = ol->name;
		olookup[q]->ol = ol;
		get_depends(ol, ol->depends);
		q++;
	}

	q = 0;
	LIST_FOREACH(cl, &chead, entries) {
		q++;
	}

	numclasses = q;

	if ((clookup = malloc(numclasses * sizeof(struct _clookup *))) == NULL)
		err(1, NULL);

	for (q = 0; q < numclasses; q++)
		if ((clookup[q] = malloc(sizeof(struct _clookup))) == NULL)
			err(1, NULL);

	q = 0;
	LIST_FOREACH(cl, &chead, entries) {
		clookup[q]->name = cl->name;
		clookup[q]->cl = cl;
		get_class_options(cl, cl->options);
		q++;
	}
}

void
free_lookups()
{
	struct classList   *cl;
	struct optionList  *ol;
	int                 q;

	for (q = 0; q < numclasses; q++)
		free(clookup[q]);

	free(clookup);

	for (q = 0; q < numoptions; q++)
		free(olookup[q]);

	free(olookup);

	while (ohead.lh_first != NULL) {	/* List Deletion. */
		ol = ohead.lh_first;
		if (ol->depends_array)
			free(ol->depends_array);

		LIST_REMOVE(ol, entries);
		free(ol);
	}

	while (chead.lh_first != NULL) {	/* List Deletion. */
		cl = chead.lh_first;
		if (cl->options_array)
			free(cl->options_array);

		LIST_REMOVE(cl, entries);
		free(cl);
	}
}

void
get_depends(struct optionList *ol, char *depends_list)
{
	int                 q;
	char               *depends_bak, **tmp_arr;

	if (!depends_list) {
		ol->depends_array = NULL;
		return;
	}
	depends_bak = strdup(depends_list);

	if ((ol->depends_array = malloc(numoptions * sizeof(char *))) == NULL)
		err(1, NULL);

	q = 0;
	tmp_arr = ol->depends_array;
	while ((*tmp_arr = strsep(&depends_bak, ",")) != NULL) {
		q++;
		tmp_arr++;
		if (q >= numoptions)	/* This _shouldn't_ happen, strictly */
			break;
	}

}

void
get_class_options(struct classList *cl, char *options_list)
{
	int                 q;
	char               *options_bak, **tmp_arr;

	if (!options_list) {
		cl->options_array = NULL;
		return;
	}
	options_bak = strdup(options_list);
	if ((cl->options_array = malloc(numoptions * sizeof(char *))) == NULL)
		err(1, NULL);

	q = 0;
	tmp_arr = cl->options_array;
	while ((*tmp_arr = strsep(&options_bak, ",")) != NULL) {
		q++;
		tmp_arr++;
		if (q >= numoptions)	/* This _shouldn't_ happen, strictly */
			break;
	}
}

void
write_makefile()
{
	struct optionList  *ol;
	struct targetList  *tl;
	struct subtList    *sl;

	LIST_HEAD           (, targetList) thead;

	int                 blanks, linepos, found;
	char               *line, *target;

	LIST_INIT(&thead);

	blanks = 0;

	LIST_FOREACH(ol, &ohead, entries) {
		if (ol->enabled) {
			fprintf(outfile, "# option %s\n", ol->name);
			while ((line = strsep(&ol->content, "\n")) != NULL) {
				if (strspn(line, " \t") == strlen(line)) {
					if (blanks++ == 0)
						fprintf(outfile, "\n");
					continue;
				}
				blanks = 0;
				if ((linepos = strcspn(line, " \t:")))
					if (line[linepos] == ':') {
						asprintf(&target, "%*.*s", linepos, linepos, line);
						if ((strncmp(target, "pre-", 4) == 0) || (strncmp(target, "post-", 5) == 0)) {
							found = 0;
							LIST_FOREACH(tl, &thead, entries) {
								if (strcmp(tl->target, target) == 0) {
									found = 1;
									break;
								}
							}
							if (!found) {
								if ((tl = malloc(sizeof(struct targetList))) == NULL)
									err(1, NULL);
								tl->target = strdup(target);
								LIST_INIT(&tl->list);
								LIST_INSERT_HEAD(&thead, tl, entries);
							}
							if ((sl = malloc(sizeof(struct subtList))) == NULL)
								err(1, NULL);
							sl->name = ol->name;
							LIST_INSERT_HEAD(&tl->list, sl, entries);
							fprintf(outfile, "%s-", ol->name);
						}
						free(target);
					}
				fprintf(outfile, "%s\n", line);
			}
		}
	}

	/* print out the targets and subtargets, deleting as we go */
	while ((tl = thead.lh_first) != NULL) {
		/* I prefix with portconf- to prevent collision */
		fprintf(outfile, "portconf-%s: ", tl->target);
		while ((sl = tl->list.lh_first) != NULL) {
			fprintf(outfile, "%s-%s ", sl->name, tl->target);
			LIST_REMOVE(sl, entries);
			free(sl);
		}
		fprintf(outfile, "\n");
		LIST_REMOVE(tl, entries);
		free(tl);
	}
}
