/*-
 *
 * Copyright (c) 1999 Neil Blakey-Milner
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <err.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "lists.h"
#include "pxml.h"
#include "portconf.h"

unsigned int
fexists(char *fname)
{
	struct stat         dummy;

	if (!lstat(fname, &dummy))
		return 1;
	return 0;
}

char               *
get_from_file(char *filename)
{
	FILE               *infile;
	int                 nchars;
	char                buffer[1024], *tmp, *tmp2;

	if ((infile = fopen(filename, "r")) == NULL)
		err(1, "%s", filename);

	if (!infile)
		return NULL;

	tmp = NULL;
	tmp2 = NULL;

	nchars = fread(buffer, 1, 1024, infile);
	asprintf(&tmp, "%s", buffer);

	while (nchars == 1024) {
		nchars = fread(buffer, 1, 1024, infile);
		asprintf(&tmp2, "%s%s", tmp, buffer);
		free(tmp);
		tmp = tmp2;
		tmp2 = NULL;
	}

	fclose(infile);
	return tmp;
}

void
portconf_init(int *argc, char *argv[])
{
	int                 ch;

	while ((ch = getopt(*argc, argv, "bc:f:")) != -1)
		switch (ch) {	/* Indent the switch. */
		case 'b':	/* Don't indent the case. */
			batch = 1;
			break;
		case 'c':
			initial_class = strdup(optarg);
			break;
		case 'f':
			makefile = strdup(optarg);
			break;
		case '?':
		default:
			usage();
			/* NOTREACHED */
		}

	(*argc) -= optind;
	argv += optind;

	directory = getcwd(NULL, MAXPATHLEN);

	if (makefile) {
		if ((outfile = fopen(makefile, "w")) == NULL)
			err(1, "%s", makefile);
	} else {
		if ((outfile = fdopen(STDERR_FILENO, "w")) == NULL)
			err(1, "stderr");
	}

	populate(PORTCONF_FILE);
	portdesc = get_from_file(COMMENTS_FILE);
}

void
portconf_end()
{
	write_makefile();
	/* Free each entry in lookup lists, and then the lists themselves */
	free_lookups();

	xmlFreeDoc(doc);
}

void
usage()
{
	fprintf(stderr, "%s%s%s\n",
	    "usage: ", (appname) ? (appname) : "",
	    " [-b] [-c class] [-f filename] portconf.xml");

	exit(1);
}
