/*-
 *
 * Copyright (c) 1999 Neil Blakey-Milner
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: pxml.c,v 1.5 1999/07/28 14:00:40 nbm Exp $
 *
 */

#include <sys/queue.h>

#include <err.h>
#include <stdlib.h>
#include <string.h>		/* for strncasecmp */

#include <tree.h>
#include <entities.h>
#include <valid.h>

#include "portconf.h"
#include "lists.h"
#include "pxml.h"

void
populate(char *filename)
{
	struct classList   *cn, *cl;
	struct optionList  *on, *ol;
	xmlNodePtr          node;

	if (!fexists(filename)) {
		errx(1, "File does not exist!\n");
		/* doesn't return */
	}
	LIST_INIT(&chead);
	LIST_INIT(&ohead);

	doc = xmlParseFile(filename);

	node = doc->root;

	portname = xmlGetProp(node, "name");

	if (strncasecmp(node->name, "portconf", 8) != 0) {
		errx(1, "%s not a port configuration file", filename);
		/* doesn't return */
	}
	node = node->childs;

	if (!node) {
		/* doesn't return */
		errx(1, "No port configuarion information in %s", filename);
	}
	while (strncasecmp(node->name, "classes", 7) != 0) {
		node = node->next;
		if (!node) {
			errx(1, "no classes (at least default is required)!");
			/* doesn't return */
		}
	}

	node = node->childs;

	while (node) {
		if (strncasecmp(node->name, "class", 5) == 0) {
			if ((cn = malloc(sizeof(struct classList))) == NULL)
				err(1, NULL);
			/* cn->name = node->properties->val->content; */
			cn->name = xmlGetProp(node, "name");
			if (!cn->name) {
				errx(1, "No name given for class");
				/* doesn't return */
			}
			cn->description = xmlGetProp(node, "description");

			if (!cn->description) {
				warnx("Option %s has no description", cn->name);
				cn->description = cn->name;
			}
			cn->options = xmlGetProp(node, "options");
			if (!cn->options)
				warnx("Class %s has no options.", cn->name);

			if (chead.lh_first == NULL) {
				LIST_INSERT_HEAD(&chead, cn, entries);
			} else {
				cl = chead.lh_first;
				while (cl->entries.le_next)
					cl = cl->entries.le_next;
				LIST_INSERT_AFTER(cl, cn, entries);
			}
		} else {
			warnx("Unexpected node (not class) \"%s\".\n",
			    xmlGetProp(node, "name"));
		}
		node = node->next;
	}

	node = doc->root->childs;

	while (strncasecmp(node->name, "options", 7) != 0) {
		node = node->next;
		if (!node) {
			errx(1, "no options!");
			/* doesn't return */
		}
	}

	node = node->childs;

	while (node) {
		if (strncasecmp(node->name, "option", 6) == 0) {
			if ((on = malloc(sizeof(struct optionList))) == NULL)
				err(1, NULL);
			on->name = xmlGetProp(node, "name");

			if (!on->name) {
				errx(1, "No name given for option");
				/* doesn't return */
			}
			on->description = xmlGetProp(node, "description");

			if (!on->description) {
				warnx("Option %s has no description", on->name);
				on->description = on->name;
			}
			on->depends = xmlGetProp(node, "depends");
			on->content = xmlNodeGetContent(node);
			on->enabled = 0;

			if (ohead.lh_first == NULL) {
				LIST_INSERT_HEAD(&ohead, on, entries);
			} else {
				ol = ohead.lh_first;
				while (ol->entries.le_next)
					ol = ol->entries.le_next;
				LIST_INSERT_AFTER(ol, on, entries);
			}
		} else {
			warnx("Unexpected node (not class) \"%s\".",
			    xmlGetProp(node, "name"));
		}
		node = node->next;
	}

	build_lookup();
}
