/*-
 *
 * Copyright (c) 1999 Neil Blakey-Milner
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include <sys/wait.h>
#include <dialog.h>

#include <stdlib.h>

#include <string.h>

#include "pdialog.h"
#include "lists.h"

#include "portconf.h"

int
getBool(dialogMenuItem *self)
{
	if (self->data && *((int *) self->data))
		return TRUE;
	return FALSE;
}

int
setBool(dialogMenuItem *self)
{
	if (self->data) {
		*((int *) self->data) = !*((int *) self->data);
		return DITEM_SUCCESS;
	}
	return DITEM_FAILURE;
}

int
button_toggle_selected(dialogMenuItem *self)
{
	struct optionList  *ol, *otmp;
	int                 q, p;

	ol = lookup_options(self->prompt);

	if (!ol) {
		fprintf(stderr, "Couldn't find lookup for %s.\n", self->prompt);
		return DITEM_FAILURE;
	}
	if (!*((int *) self->data)) {
		/* turning the thingy on */
		*((int *) self->data) = !*((int *) self->data);
		if (!ol->depends_array)
			return DITEM_SUCCESS;

		for (p = 0; ol->depends_array[p]; p++) {
			otmp = lookup_options(ol->depends_array[p]);
			if (!*((int *) ((dialogMenuItem *)otmp->widget)->data)) {
				/*
				 * recursively turn on and check children's
				 * children
				 */
				button_toggle_selected(otmp->widget);
			}
		}
	} else {
		/* otherwise, turning it off */
		*((int *) self->data) = !*((int *) self->data);
		for (q = 0; q < numoptions; q++) {
			/*
			 * search each optionList for this name in it's
			 * depends list
			 */
			otmp = olookup[q]->ol;
			if (otmp->depends_array == NULL)
				continue;

			for (p = 0; otmp->depends_array[p]; p++) {
				if ((strcasecmp(otmp->depends_array[p], self->prompt) == 0)
				    && ((*((int *) ((dialogMenuItem *)otmp->widget)->data)))) {
					/*
					 * recursively turn off and check
					 * children's children
					 */
					button_toggle_selected(otmp->widget);
				}
			}
		}
	}
	return DITEM_SUCCESS | DITEM_REDRAW;
}

int
class_select(dialogMenuItem *self)
{
	struct classList   *cl;
	struct optionList  *ol;
	int                 p;

	cl = lookup_class(self->prompt);
	if (!cl) {
		fprintf(stderr, "No such class %s!\n", self->prompt);
		return DITEM_FAILURE;
	}
	LIST_FOREACH(ol, &ohead, entries) {
		ol->enabled = 0;
	}

	if (cl->options_array) {
		for (p = 0; cl->options_array[p]; p++) {
			ol = lookup_options(cl->options_array[p]);
			if (!ol)
				break;

			button_toggle_selected(ol->widget);
		}
	}
	return DITEM_SUCCESS | DITEM_REDRAW;
}

void
dialog_stuff(void)
{
	struct optionList  *ol;
	struct classList   *cl;
	dialogMenuItem     *options_menu;
	int                 p, retval;

	options_menu = malloc((numoptions + numclasses + 1) * sizeof(dialogMenuItem));

	p = 0;

	LIST_FOREACH(cl, &chead, entries) {
		options_menu[p].prompt = cl->name;
		options_menu[p].title = cl->description;
		options_menu[p].checked = NULL;
		options_menu[p].fire = class_select;
		options_menu[p].selected = NULL;
		options_menu[p].data = NULL;
		options_menu[p].lbra = ' ';
		options_menu[p].mark = ' ';
		options_menu[p].rbra = ' ';
		p++;
	}

	options_menu[p].prompt = "--------";
	options_menu[p].title = "-------------------------------";
	options_menu[p].checked = NULL;
	options_menu[p].fire = NULL;
	options_menu[p].selected = NULL;
	options_menu[p].data = NULL;
	options_menu[p].lbra = ' ';
	options_menu[p].mark = ' ';
	options_menu[p].rbra = ' ';
	p++;

	LIST_FOREACH(ol, &ohead, entries) {
		ol->widget = &(options_menu[p]);
		options_menu[p].prompt = ol->name;
		options_menu[p].title = ol->description;
		options_menu[p].checked = getBool;
		options_menu[p].fire = button_toggle_selected;
		options_menu[p].selected = NULL;
		options_menu[p].data = &(ol->enabled);
		options_menu[p].lbra = NULL;
		options_menu[p].mark = NULL;
		options_menu[p].rbra = NULL;
		p++;
	}

	init_dialog();

	dialog_clear();

	retval = dialog_checklist(portname, portdesc, -1, -1,
	    ((numoptions + numclasses + 1) < 15) ? (numoptions + numclasses + 1) : 15,
	    -(numoptions + numclasses + 1),
	    options_menu, NULL);

	free(options_menu);

	end_dialog();
}
