/*-
 *
 * Copyright (c) 1999 Neil Blakey-Milner
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include <gtk/gtk.h>

#include <sys/queue.h>

#include <stdlib.h>
#include <unistd.h>
#include <string.h>		/* for strncasecmp */

#include "portconf.h"
#include "lists.h"
#include "pxml.h"

static void
exit_app(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	//gtk_exit(1);
	gtk_main_quit();
}

static gint
delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	//g_print("delete event occurred\n");
	return (FALSE);
}

static void
destroy(GtkWidget *widget, gpointer data)
{
	gtk_main_quit();
}

static GtkWidget   *
create_YesNo_box()
{
	GtkWidget          *YesNo_hbutton_box, *CancelButton, *OkButton;

	YesNo_hbutton_box = gtk_hbutton_box_new();
	CancelButton = gtk_button_new_with_label("Cancel");
	gtk_signal_connect_object(GTK_OBJECT(CancelButton), "clicked",
	    GTK_SIGNAL_FUNC(exit_app), NULL);
	OkButton = gtk_button_new_with_label("OK");
	gtk_container_add(GTK_CONTAINER(YesNo_hbutton_box), CancelButton);
	gtk_container_add(GTK_CONTAINER(YesNo_hbutton_box), OkButton);

	gtk_widget_show(CancelButton);
	gtk_widget_show(OkButton);
	return YesNo_hbutton_box;
}

static GtkWidget   *
create_portdescription()
{
	GtkWidget          *DescrFrame, *DescrText, *vscrollbar, *DescrBox;

	DescrFrame = gtk_frame_new("Port Description");
	gtk_container_set_border_width(GTK_CONTAINER(DescrFrame), 5);
	DescrText = gtk_text_new(NULL, NULL);
	gtk_text_set_word_wrap(GTK_TEXT(DescrText), TRUE);

	gtk_text_insert(GTK_TEXT(DescrText), NULL, NULL, NULL, portdesc, strlen(portdesc));

	DescrBox = gtk_hbox_new(FALSE, 0);
	vscrollbar = gtk_vscrollbar_new(GTK_TEXT(DescrText)->vadj);
	gtk_container_add(GTK_CONTAINER(DescrBox), DescrText);
	gtk_box_pack_start(GTK_BOX(DescrBox), vscrollbar, FALSE, FALSE, 0);

	gtk_container_add(GTK_CONTAINER(DescrFrame), DescrBox);
	gtk_container_set_border_width(GTK_CONTAINER(DescrFrame), 2);
	gtk_widget_show(DescrText);
	gtk_widget_show(DescrBox);
	gtk_widget_show(vscrollbar);

	return DescrFrame;
}

static void
class_button_clicked(GtkWidget *button, char *name)
{
	struct classList   *cl;
	struct optionList  *on;
	int                 p;

	cl = lookup_class(name);
	if (!cl) {
		fprintf(stderr, "No such class %s!\n", name);
		return;
	}
	LIST_FOREACH(on, &ohead, entries) {
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(on->widget), FALSE);
	}

	if (cl->options_array) {
		for (p = 0; cl->options_array[p]; p++) {
			on = lookup_options(cl->options_array[p]);
			if (!on)
				break;
			if (!(GTK_TOGGLE_BUTTON(on->widget)->active)) {
				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(on->widget), TRUE);
				gtk_toggle_button_toggled(GTK_TOGGLE_BUTTON(on->widget));
			}
		}
	}
}

static GtkWidget   *
create_ClassesFrame(char *title)
{
	struct classList   *cn;
	GtkWidget          *ClassFrame, *classes_hbutton_box, *classesbuttons;
	GtkTooltips        *tooltips;

	ClassFrame = gtk_frame_new(title);
	classes_hbutton_box = gtk_hbutton_box_new();
	gtk_container_add(GTK_CONTAINER(ClassFrame), classes_hbutton_box);

	LIST_FOREACH(cn, &chead, entries) {
		classesbuttons = gtk_button_new_with_label(cn->name);
		tooltips = gtk_tooltips_new();
		gtk_signal_connect(GTK_OBJECT(classesbuttons), "clicked",
		    GTK_SIGNAL_FUNC(class_button_clicked), cn->name);
		gtk_tooltips_set_tip(tooltips, classesbuttons, cn->description, NULL);
		gtk_container_add(GTK_CONTAINER(classes_hbutton_box), classesbuttons);
		gtk_widget_show(classesbuttons);
	}

	gtk_widget_show(classes_hbutton_box);
	return ClassFrame;
}

static void
button_toggle_selected(GtkWidget *checkbutton, char *name)
{
	struct optionList  *ol, *otmp;
	int                 p, q;

	ol = lookup_options(name);

	if (!ol) {
		fprintf(stderr, "Couldn't find lookup for %s.\n", name);
		return;
	}
	if (GTK_TOGGLE_BUTTON(checkbutton)->active) {
		/* turning the thingy on */
		ol->enabled = 1;
		if (ol->depends_array) {
			for (p = 0; ol->depends_array[p]; p++) {
				otmp = lookup_options(ol->depends_array[p]);
				/*
				 * printf ("Trying to turn %s on\n",
				 * otmp->name);
				 */
				if (!(GTK_TOGGLE_BUTTON(otmp->widget)->active)) {
					gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(otmp->widget), TRUE);
					gtk_toggle_button_toggled(GTK_TOGGLE_BUTTON(otmp->widget));
				}
			}
		}
	} else {
		/* otherwise, turning it off */
		ol->enabled = 0;
		for (q = 0; q < numoptions; q++) {
			/*
			 * search each optionList for this name in it's
			 * depends list
			 */
			otmp = olookup[q]->ol;
			if (otmp->depends_array != NULL) {
				/*
				 * fprintf(stderr, "Options %s's
				 * depends_array is not empty\n", ol->name);
				 */

				for (p = 0; otmp->depends_array[p]; p++) {
					if (strcasecmp(otmp->depends_array[p], name) == 0) {
						if ((GTK_TOGGLE_BUTTON(otmp->widget)->active)) {
							gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(otmp->widget), FALSE);
							gtk_toggle_button_toggled(GTK_TOGGLE_BUTTON(otmp->widget));
						}
						/*
						 * printf ("Should turn %s
						 * off\n", otmp->name);
						 */
					}
				}
			}
		}
	}
}

static GtkWidget   *
create_OptionsFrame(char *title)
{
	struct optionList  *n1;
	GtkWidget          *OptFrame, *OptionsTable, *checkbox;
	GtkWidget          *labels, *depends;
	int                 q;

	OptFrame = gtk_frame_new(title);

	OptionsTable = gtk_table_new(numoptions, 3, FALSE);
	gtk_container_set_border_width(GTK_CONTAINER(OptionsTable), 2);
	gtk_container_add(GTK_CONTAINER(OptFrame), OptionsTable);

	q = 0;
	LIST_FOREACH(n1, &ohead, entries) {
		checkbox = gtk_check_button_new_with_label(n1->name);
		labels = gtk_label_new(n1->description);
		depends = gtk_label_new(n1->depends);

		gtk_signal_connect(GTK_OBJECT(checkbox), "toggled",
		    GTK_SIGNAL_FUNC(button_toggle_selected), n1->name);

		gtk_label_set_justify(GTK_LABEL(labels), GTK_JUSTIFY_RIGHT);
		gtk_table_attach(GTK_TABLE(OptionsTable), checkbox,
		    0, 1, q, q + 1,
		    GTK_EXPAND | GTK_SHRINK | GTK_FILL,
		    GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
		gtk_table_attach(GTK_TABLE(OptionsTable), labels,
		    1, 2, q, q + 1,
		    GTK_EXPAND | GTK_SHRINK | GTK_FILL,
		    GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
		gtk_table_attach(GTK_TABLE(OptionsTable), depends,
		    2, 3, q, q + 1,
		    GTK_EXPAND | GTK_SHRINK | GTK_FILL,
		    GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
		gtk_widget_show(checkbox);
		gtk_widget_show(depends);
		gtk_widget_show(labels);
		n1->widget = checkbox;
		q++;
	}

	gtk_widget_show(OptionsTable);
	return OptFrame;
}

void
gtk_stuff()
{
	GtkWidget          *window, *main_container_box, *YesNo_box;
	GtkWidget          *ClassesFrame, *OptionsFrame, *DescriptionFrame;

	window = gtk_window_new(GTK_WINDOW_DIALOG);
	gtk_window_set_title(GTK_WINDOW(window), "Port configuration");
	gtk_window_set_policy(GTK_WINDOW(window), 0, 0, 1);
	GTK_WINDOW(window)->type = GTK_WINDOW_DIALOG;
	gtk_container_set_border_width(GTK_CONTAINER(window), 5);

	gtk_signal_connect(GTK_OBJECT(window), "delete_event",
	    GTK_SIGNAL_FUNC(delete_event), NULL);

	gtk_signal_connect(GTK_OBJECT(window), "destroy",
	    GTK_SIGNAL_FUNC(destroy), NULL);

	main_container_box = gtk_vbox_new(FALSE, 3);

	ClassesFrame = create_ClassesFrame("Predefined Classes");
	OptionsFrame = create_OptionsFrame("Available Options");
	YesNo_box = create_YesNo_box();
	DescriptionFrame = create_portdescription();

	gtk_widget_show(YesNo_box);
	gtk_widget_show(DescriptionFrame);
	gtk_widget_show(ClassesFrame);
	gtk_widget_show(OptionsFrame);

	gtk_container_add(GTK_CONTAINER(window), main_container_box);
	gtk_container_add(GTK_CONTAINER(main_container_box), DescriptionFrame);
	gtk_container_add(GTK_CONTAINER(main_container_box), ClassesFrame);
	gtk_container_add(GTK_CONTAINER(main_container_box), OptionsFrame);
	gtk_container_add(GTK_CONTAINER(main_container_box), YesNo_box);

	gtk_widget_show(main_container_box);
	gtk_widget_show(window);
}
