#!/usr/bin/perl
#
# Copyright (c) 1999 Neil Blakey-Milner
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# $Id: portconf.pl,v 1.2 1999/08/22 13:59:52 nbm Exp $
#

use strict;
use Getopt::Std;
use POSIX qw(tmpnam);
use Fcntl qw(O_CREAT O_RDWR O_EXCL);

getopts('bc:f:');

my $context;
my $optname;
my $classname;

my %contents;
my %classopts;
my %depends;

my %optdesc;
my %classdesc;

my @options;
my @classes;

my $chosenopts;
my %chosenhash;

my $initclass = $Getopt::Std::opt_c || 'default';

# contexts:
#  0 - nothing
#  1 - in portconf
#  2 - classes
#  3 - waiting for options
#  4 - got options
#  5 - in an option
#  6 - closure

open(PORTCONF_FILE, "files/portconf.xml") or die "Can't open portconf file!";

if ($Getopt::Std::opt_f) {
	open(OUT, ">$Getopt::Std::opt_f") or die "Can't open make file!";
} else {
	open(OUT, ">&STDERR");
}

$context = 0;

while (&getline ne "-1") {
	if ($context != 5) {
		die "no preceeding <!\n" if (substr($_,0,1) ne "<");
		my $backup;
		$backup = $_;
		while ((substr($_, -1, 1) ne ">") and (&getline)) {
			$backup .= " $_";
		}
		$_ = $backup;
		s/ *= */=/;
	}
	#print "$_\n";
		
	if ($context == 0) {
		/<portconf(?: name="([^"]*)")/i || die "not a portconf file\n";
		#print "Portconf name is $1!\n";
		$context = 1;
		next;
	}
	if ($context == 1) {
		/<classes/i || die "no classes after portconf?!\n";
		$context = 2;
		next;
	}
	if ($context == 2) {
		/<\/classes>/i && do { $context = 3; next };
		/<class name="([^"]*)"(?: description="([^"]*)")(?: options="([^"]*)")/i || die "non-class statement in classes section!\n";
		$classname = $1;
		$classopts{$classname} = $3;
		$classdesc{$classname} = $2;
		push @classes, $classname;
		#print "Class name is $1 with description $2!\n";
		#if ($3) {
		#	print "Options are $3\n";
		#}
		next;
	}
	if ($context == 3) {
		/<options/i || die "no options?!\n";
		$context = 4;
		next;
	}
	if ($context == 4) {
		/<\/options>/i && do { $context = 6; next };
		/<option name="([^"]*)"(?: description="([^"]*)")(?: depends="([^"]*)")?/i || die "non-option in options section!\n";
		$context = 5; # scary!
		$optname = $1;
		$contents{$optname} = "";
		$optdesc{$optname} = $2;
		$depends{$optname} = $3;
		push @options, $optname;
		next;
	}
	if ($context == 5) {
		/<\/option>/i && do {
			#print "End of option $optname!\n";
			$context = 4; next 
		};
		$contents{$optname} .= "$_\n";
		next;
	}
	if ($context == 6) {
		/<\/portconf>/i && last;
		die "unexpected statement before end-of-file!\n";
	}
}

if ($context != 6) {
	die "didn't finish getting configuration!\n";
}

my $foo;
my $opt;

#$chosenopts = join(' ', split /,/, $classopts{'default'});
($chosenopts = $classopts{$initclass}) =~ tr/,/ /;

unless ($Getopt::Std::opt_b) {
	$foo = '"\nPlease select desired options: (Esc to reset, Cancel to quit)" 20 70 13 ';

	foreach $opt (@options) {
		$foo .= "$opt \"";
		$foo .= $optdesc{$opt} || $opt;
		if ($chosenopts =~ /\b$opt\b/) {
			$foo .= "\" ON ";
		} else {
			$foo .= "\" OFF ";
		}
	}

	my $tmpfile;
	$tmpfile = mktmpfile();

	while(system("/bin/sh", "-c", "dialog --title \"configuration options\" --clear --checklist $foo 2>$tmpfile") != 0) {
		die "Cancelled $?\n" if ($? == 256);
	}

	open(TMP, $tmpfile);
	chomp($chosenopts = <TMP>);
	close(TMP);
	unlink($tmpfile);
	$chosenopts =~ s/"//g;
}

foreach $opt (split / /, $chosenopts) {
	$chosenhash{$opt} = 1;
}

DEPS: foreach $opt (keys %chosenhash) {
	foreach (split / /, $depends{$opt}) {
		unless($chosenhash{$_}) {
			$chosenhash{$_} = 1;
			goto DEPS;
		}
	}
}

my @output;
my %localtargets;

foreach $opt (keys %chosenhash) {
	foreach (split /\n/, $contents{$opt}) {
		if (/((?:pre|post)-[^ ]+):/) {
			if (!defined($localtargets{$1})) {
				$localtargets{$1} = "$opt";
			} else {
				$localtargets{$1} .= " $opt";
			}
			$_ = "$opt-$_";
		}
		s/&amp;/&/g;
		s/&gt;/>/g;
		s/&lt;/>/g;
		print OUT "$_\n";
	}
}

my $target;

foreach $target (keys %localtargets) {
	print OUT "portconf-$target: ";
	foreach (split / /, $localtargets{$target}) {
		print OUT "$_-$target ";
	}
	print OUT "\n";
}

exit;

sub getline {
	return "-1" unless ($_ = <PORTCONF_FILE>);
	chomp;
	if ($context != 5) {
		s/\t/ /g;
		s/ +/ /g;
		s/^ +//g;
		s/ +$//g;
		&getline if ($_ eq "");
	}
	return $_;
}

sub mktmpfile {
	my ($tmpfile);
	do { $tmpfile = tmpnam() }
		until (sysopen(TADA, $tmpfile, O_CREAT | O_RDWR | O_EXCL));
	close TADA;
	return $tmpfile;
}
